<?php
/**
 * Developped for Stip by Yolknet Internet Services
 * @author Dirk Kokx (dirk@yolknet.nl)
 */

// No direct access to this file
defined('_JEXEC') or die('Restricted access');

// Load the language files
$app = JFactory::getApplication();
$app->getLanguage()->load('lib_easylib', JPATH_ROOT);

/**
 * Contains functions to prepare the form to show EasyImage Field
 * Class StipEasyimageFormHelper
 */
class StipEasyimageFormHelper {

    /**
     * Holds the current group the fields belong to
     * @var    string
     */
    private $fieldGroup = '';
    private $fieldPath = false;
    private $ajaxUrl = '';


    /**
     * @param JForm $form
     * @param bool $with_select
     * @param array $change_fields [
     *   'FIELDNAME' => [
     *          attributes => [ATTRIBUTE_NAME => ATTRIBUTE_VALUE...],
     *          group => GROUPNAME
     *   ], ...
     * ]
     * @param array $hide_fields [
     *      [
     *          name => FIELDNAME,
     *          group => GROUPNAME,
     *      ], ...
     * ]
     */
    public function setFormFields(JForm &$form, &$data, $with_select, $ajax_url, $change_fields = [], $hide_fields = [])
    {
        $this->setAjaxPath($ajax_url);
        if(count($change_fields)) {
            foreach ($change_fields as $fieldName => $changeData) {
                $this->fieldGroup = $changeData['group'];
                $field = $form->getField($fieldName, $this->fieldGroup);
                if(!$field)
                    continue;

                foreach ($changeData['attributes'] as $name => $value) {
                    $form->setFieldAttribute(
                        (string)$fieldName,
                        $name,
                        $value,
                        $this->fieldGroup
                    );
                }

                //change to easyimage
                $this->changeToStipImage($form, $fieldName);

                if ($with_select)
                    $this->addSelectField($form, $data, (string) $this->fieldGroup, (string) $fieldName);
            }
        }

        if (count($hide_fields)) {
            foreach ($hide_fields as $hide_field) {
                $form->setFieldAttribute(
                    $hide_field['name'],
                    'type',
                    'hidden',
                    $hide_field['group']
                );
            }
        }
    }


    /**
     * cleans the url path if neccessary
     * @param $path
     * @throws Exception
     */
    private function setAjaxPath($path)
    {
        if(substr($path, 0, 4) == 'http')
        {
            //we only allow paths relative to the current website
            list($pfx, $path) = explode('index.php', $path);
            if(! $path){
                throw new Exception(JText::_('LIB_EASYLIB_EASYIMAGE_ERR_INVALID_AJAXURL_START_WITH_INDEX'));
            } else {

            }
            $this->ajaxUrl = $path;
        } elseif (substr($path, 0, 10) == '/index.php'){
            $this->ajaxUrl = substr($path, 1);
        } elseif (substr($path, 0, 9) == 'index.php') {
            $this->ajaxUrl = $path;
        } else {
            throw new Exception(JText::_('LIB_EASYLIB_EASYIMAGE_ERR_INVALID_AJAXURL'));
        }
    }

    /**
     * additional field that gets triggered by the select button to open the filebrowser
     * @param $form
     * @param $data (object or array)
     * @param $groupName
     * @param $origName
     */
    public function addSelectField(&$form, &$data, $groupName, $origName)
    {
        $form->load('
            <form>
                <fields name="'.$groupName.'">
                    <fieldset>
                        <field
                            name="stip_select_'.$origName.'"
                            type="media"
                            accept="image/*"
                            preview="false"
                            id="stip_select_'.$origName.'"
                            class="stip_select"
                        />
                    </fieldset>
                </fields>
            </form>
        ');

        if(is_array($data)){
            if(isset($data[$groupName])){
                $groupdata = (is_array($data[$groupName]))? $data[$groupName] : json_decode($data[$groupName], 1);
                if($groupdata[$origName])
                    $groupdata["stip_select_".$origName] = $groupdata[$origName];

                $data[$groupName] = (is_array($data[$groupName]))? $groupdata : json_encode($groupdata);
            }
        } else {
            if(isset($data->$groupName)){
                $groupdata = (is_array($data->$groupName))? $data->$groupName : json_decode($data->$groupName, 1);
                if(isset($groupdata[$origName]))
                    $groupdata["stip_select_".$origName] = $groupdata[$origName];

                $data->$groupName = (is_array($data->$groupName))? $groupdata : json_encode($groupdata);
            }
        }
    }

    private function changeToStipImage(JForm &$form, $fieldName)
    {

        $form->setFieldAttribute(
            (string) $fieldName,
            'type',
            "easyimage",
            $this->fieldGroup
        );

        $form->setFieldAttribute(
            (string) $fieldName,
            'ajaxurl',
            $this->ajaxUrl,
            $this->fieldGroup
        );

        $form->setFieldAttribute(
            (string) $fieldName,
            'accept',
            "image/*",
            $this->fieldGroup
        );

        if(! $this->fieldPath) {
            $form->setFieldAttribute(
                (string) $fieldName,
                'addfieldpath',
                "libraries/easylib/forms/fields",
                $this->fieldGroup
            );

            $this->fieldPath = true;
        }
    }

    public static function setAssets($custom_js = false)
    {

        if($custom_js){
            $doc = JFactory::getDocument();
            $doc->addScriptDeclaration($custom_js);
        }

    }

    public static function processAjax($quality = 90)
    {
        $jinput = JFactory::getApplication()->input;
        $file = $jinput->files->get('file');
        $filename = $jinput->getString('filename');
        $type = $jinput->get('type');
        $width = $jinput->get('width');
        $height = $jinput->get('height');
        $folder = $jinput->get('folder', null, 'RAW');

        try {
            $easyImage = new StipEasyImage($file, $type);
        } catch (Exception $e){
            return $e;
        }

        return $easyImage->saveImage($width, $height, $filename, $folder, $quality);

    }
}
