<?php
/**
 * Developped for Stip by Yolknet Internet Services
 * @author Dirk Kokx (dirk@yolknet.nl)
 */

defined('_JEXEC') or die;

jimport('joomla.form.formfield');

// Include jQuery
JHtml::_('behavior.modal');
JHtml::_('jquery.framework');
JHtml::_('stylesheet', 'easylib/stipeasyimage.min.css', ['version' => 'auto', 'relative' => true]);
JHtml::_('stylesheet', 'easylib/cropper.min.css', ['version' => 'auto', 'relative' => true]);
JHtml::_('script', 'media/easylib/js/jquery/cropper.min.js');

/**
 * Class JFormFieldUpload
 *
 * The field has these params:
 * name (string, name of input)
 * addfieldpath="libraries/stip/forms/fields" - required to get to the field opbject
 * type="easyimage"
 * ajaxurl="index.php?option=com_ajax...format=json" - define where the ajax upload should be handled
 * label (string)
 * description (string)
 * accept  (optional, defaults to "image/*")
 * show_select (bool, whether or not to show the select-button) - when true a select-input of type media should exist with class 'stip_select' and a name "stip_select_"+the name of this field
 * copy_to (string, optional, name of other inputfield that should inherit the same image)
 *
 * the field accepts sub-elements of type 'size', with the following params:
 * width (int or 0)
 * height (int or 0)
 * folder' (string, path to the folder where the image should be uploaded)
 * name (string, optional, the name of the size-setting)
 * param_key (int, optional id of the size-setting)
 * cancrop (bool whether or not to show the pop-up for cropping, optional) defaults to true when height and width arre both > 0
 * autopopup (bool whether or not to show the cropping-popup immediately after adding a file, optional)
 */
class JFormFieldEasyImage extends JFormField {

    //The field class must know its own type through the variable $type.
    protected $type = 'EasyImage';

    public function getLabel() {
        // code that returns HTML that will be shown as the label

        if (!empty($this->element['description']))
        {
            JHtml::_('bootstrap.popover');
            $title     = ' title="' . htmlspecialchars(trim($this->element['label'], ':')) . '"'
                . ' data-content="'. htmlspecialchars($this->element['description']) . '"';

            return '<label class="easyimage_label hasPopover"'.$title.'>'.$this->element['label'].'</label>';
        } else {

            return '<label class="easyimage_label">'.$this->element['label'].'</label>';

        }
    }

    public function getInput() {

        $app = JFactory::getApplication();
        $app->getLanguage()->load('lib_easylib', JPATH_ROOT);
        $is_admin = ($app->isClient('administrator'))? "true" : "false";

        $accept = isset($this->element['accept'])?
            $this->element['accept'] :
            "image/*";

        $sizes = $this->getSizes();
        $copy_to = isset($this->element['copy_to'])? (string) $this->element['copy_to'] : 0;
        $multiple = isset($this->element['multiple'])? (int) $this->element['multiple'] : 0;
        $select = isset($this->element['show_select']) ? (int)$this->element['show_select'] : 0;
        $ajaxurl = $this->element['ajaxurl'];

        // code that returns HTML that will be shown as the form field
        $class = ($multiple)? "uploadresize multifield" : "uploadresize";
        $output = '<div class="'.$class.'" data-accept="'.$accept.'" data-copy_to="'.$copy_to.'" data-ajaxurl="'.JURI::root().$ajaxurl.'" data-isadmin="'.$is_admin.'" data-baseurl="'.JUri::base().'" data-select="'.$select.'" data-multiple="'.$multiple.'" data-fieldname="'.$this->element['name'].'">
        <div class="sizes hidden">'.$sizes.'</div>
        <input 
            type="file"
	        name="'.$this->element['name'].'"
        	id="'.$this->element['name'].'"';

        if((bool) $multiple)
            $output .= ' multiple ';

        $output .= ' accept="'.$accept.'"/>';

        $output .= '<input type="hidden" id="'.$this->id.'" name="' . $this->name. '" value="' . $this->value . '" />';

        if($multiple){
            $choose = JText::_("LIB_EASYLIB_EASYIMAGE_CHOOSE_FILES");
            $upload = JText::_("LIB_EASYLIB_EASYIMAGE_OR_DRAG_FILES");
        } else {
            $choose = JText::_("LIB_EASYLIB_EASYIMAGE_CHOOSE_FILE");
            $upload = JText::_("LIB_EASYLIB_EASYIMAGE_OR_DRAG_FILE");
        }
        $or = JText::_('LIB_EASYLIB_EASYIMAGE_OR');
        $cancel = JText::_("LIB_EASYLIB_EASYIMAGE_CANCEL");
        $save = JText::_("LIB_EASYLIB_EASYIMAGE_SAVE");
        $select_size = JText::_("LIB_EASYLIB_EASYIMAGE_SELECT_SIZE");
        $select_text = JText::_('LIB_EASYLIB_EASYIMAGE_SELECT');

        $output .= '<div class="display">
                        <div class="explanation">
                        <strong>'.$upload.'</strong><span class="or">'.$or.'</span>
                        <label for="'.$this->element['name'].'">'.$choose.'</label> </div></div>';

        if(! (bool) $multiple || $select){
            $output .= '<div class="editremove">';

            if(! (bool) $multiple) {
                $output .= '<span class="removebutton"><i class="icon-remove"></i></span>';
                $output .= '<span class="editbutton hidden"><i class="icon-edit"></i>Edit</span>';
            }

            if($select)
                $output .= '<div class="selectbutton" style="display: inline-block;">'.$select_text.'</div>';

            $output .= '</div>';
        }
        $output .= '</div>';
        $output .= '<div class="resizemodal '.$this->element['name'].'"><div class="canvas"><img src="" style="max-width: 100%"/></div><div class="button-row"> <div class="closebutton"><i class="icon-remove"></i>'.$cancel.'</div> <div class="submitbutton"><i class="icon-checkmark-2"></i>'.$save.'</div></div></div>';
        $output .= '<div class="sizesmodal '.$this->element['name'].'"><div class="modal_screen"><div class="header"><h3>'.$select_size.'</h3><div class="close-button"><i class="icon-remove"></i></div><div class="clearfix"></div></div><div class="body"></div></div></div>';

        if((bool) $multiple){
            $output .= '<div class="multiple"></div>';
            JHtml::_('script', 'media/easylib/js/stipeasyimage_multiple.min.js');
        } else {
            JHtml::_('script', 'media/easylib/js/stipeasyimage.min.js');
        }
        return $output;
    }

    protected function getSizes()
    {
        $sizes = (string) $this->element['sizes'];

        if (!$sizes || $sizes == "") {
            $sizes = [];
            $i = 1;
            foreach ($this->element->xpath('size') as $size){
                $tmp = [];
                $tmp['id'] = $i;
                $tmp['width'] = (isset($size['width']))? (int) $size['width'] : 0;
                $tmp['height'] = (isset($size['height']))? (int) $size['height'] : 0;
                $tmp['folder'] = (string) $size['folder'];

                if(isset($size['name']))
                    $tmp['name'] = (string) $size['name'];

                if($tmp['width'] > 0 && $tmp['height'] > 0) {
                    if (isset($size['cancrop'])) {
                        $tmp['cancrop'] = (bool) (int) $size['cancrop'];
                    } else {
                        $tmp['cancrop'] = true;
                    }
                }

                $tmp['autopopup'] = (isset($size['autopopup']))? (bool) (int) $size['autopopup']: true;
                $tmp['allow_upscale'] = (isset($size['allow_upscale']))? (bool) (int) $size['allow_upscale']: true;

                $sizes[] = $tmp;

                $i++;
            }

            $sizes = json_encode($sizes);
        }

        return $sizes;
    }

}
